import pandas as pd
import requests as req

from collections import defaultdict
from functools import partial, reduce
from multiprocessing import Pool

from statsbombpy import api_client, public
from statsbombpy.config import DEFAULT_CREDS, PARALLELL_CALLS_NUM
from statsbombpy.helpers import filter_and_group_events, is_relevant, reduce_events


def competitions(fmt="dataframe", creds: dict = DEFAULT_CREDS):
    if api_client.has_auth(creds) is True:
        competitions = api_client.competitions(creds)
    else:
        competitions = public.competitions()
    if fmt == "dataframe":
        if isinstance(competitions, dict):
            competitions = competitions.values()
        competitions = pd.DataFrame(competitions)
    return competitions


def matches(
    competition_id: int, season_id: int, fmt="dataframe", creds: dict = DEFAULT_CREDS
):
    if api_client.has_auth(creds) is True:
        matches = api_client.matches(competition_id, season_id, creds)
    else:
        matches = public.matches(competition_id, season_id)
    if fmt == "dataframe":
        matches = pd.DataFrame(matches.values())
        matches["competition"] = matches.competition.apply(
            lambda c: f"{c['country_name']} - {c['competition_name']}"
        )
        for col in ["season", "home_team", "away_team"]:
            matches[col] = matches[col].apply(lambda c: c[f"{col}_name"])
        for col in ["competition_stage", "stadium", "referee"]:
            matches[col] = matches[col].apply(
                lambda x: x["name"] if not pd.isna(x) else x
            )
        metadata = matches.pop("metadata")
        for k in ["data_version", "shot_fidelity_version", "xy_fidelity_version"]:
            matches[k] = metadata.apply(lambda x: x.get(k))
    return matches


def lineups(match_id, fmt="dataframe", creds: dict = DEFAULT_CREDS):
    if api_client.has_auth(creds) is True:
        lineups = api_client.lineups(match_id, creds)
    else:
        lineups = public.lineups(match_id)
    if fmt == "dataframe":
        lineups_ = {}
        for l in lineups.values():
            lineup = pd.DataFrame(l["lineup"])
            lineup["country"] = lineup.country.apply(lambda c: c["name"])
            lineups_[l["team_name"]] = lineup
            lineups = lineups_
    return lineups


def events(
    match_id: int,
    split: bool = False,
    filters: dict = {},
    fmt: str = "dataframe",
    flatten: bool = False,
    creds: dict = DEFAULT_CREDS,
) -> (pd.DataFrame, dict):

    if api_client.has_auth(creds) is True:
        events = api_client.events(match_id, creds)
    else:
        events = public.events(match_id)

    if fmt == "dataframe":
        events = filter_and_group_events(events, filters, fmt, flatten)
        for ev_type, evs in events.items():
            events[ev_type] = pd.DataFrame(evs)
        if split is False:
            events = pd.concat([*events.values()], axis=0, ignore_index=True, sort=True)
    return events


def competition_events(
    country: str,
    division: str,
    season: str,
    gender: str = "male",
    split: bool = False,
    filters: dict = {},
    fmt: str = "dataframe",
    creds: dict = DEFAULT_CREDS,
) -> (pd.DataFrame, dict):

    c = competitions(creds)[country, division, season, gender]

    events_call = partial(events, fmt="json", creds=creds,)
    with Pool(PARALLELL_CALLS_NUM) as p:
        matches_events = map(
            events_call, matches(c["competition_id"], c["season_id"], creds)
        )
        matches_events = map(
            lambda events: filter_and_group_events(
                events, filters, fmt, fmt == "dataframe"
            ),
            matches_events,
        )

    competition_events = reduce_events(matches_events, fmt)
    if fmt == "dataframe" and split is False:
        competition_events = pd.concat(
            [*competition_events.values()], axis=0, ignore_index=True, sort=True
        )
    return competition_events



import requests as req

import statsbombpy.entities as ents

from statsbombpy.config import OPEN_DATA_PATHS


def competitions():
    competitions = req.get(OPEN_DATA_PATHS["competitions"]).json()
    competitions = ents.competitions(competitions)
    return competitions


def matches(competition_id: int, season_id: int) -> dict:
    matches = req.get(
        OPEN_DATA_PATHS["matches"].format(
            competition_id=competition_id, season_id=season_id
        )
    ).json()
    matches = ents.matches(matches)
    return matches


def lineups(match_id: int):
    lineups = req.get(OPEN_DATA_PATHS["lineups"].format(match_id=match_id)).json()
    lineups = ents.lineups(lineups)
    return lineups


def events(match_id: int) -> dict:
    events = req.get(OPEN_DATA_PATHS["events"].format(match_id=match_id)).json()
    events = ents.events(events, match_id)
    return events



def competitions(competitions: list) -> dict:
    competitions_ = {}
    for c in competitions:
        competitions_[
            (
                c["country_name"],
                c["competition_name"],
                c["season_name"],
                c["competition_gender"],
            )
        ] = c
    return competitions_


def matches(matches: list) -> dict:
    matches_ = {
        m["match_id"]: m
        for m in matches
        if m["match_status"] == "available"
    }
    return matches_


def lineups(lineups: list) -> dict:
    lineups_ = {l["team_id"]: l for l in lineups}
    return lineups_


def events(events: list, match_id: int) -> dict:
    events_ = {}
    for e in events:
        e["match_id"] = match_id
        events_[e["id"]] = e
    return events_


from setuptools import setup


setup(
    name="statsbombpy",
    version="0.1",
    description="easily stream StatsBomb data into Python",
    url="https://github.com/statsbomb/statsbombpy",
    author="StatsBomb",
    author_email="support@statsbombservices.com",
    packages=["statsbombpy"],
    install_requires=["cashier", "inflect", "nose2", "pandas", "requests", "requests-cache"],
)



from collections import defaultdict

import inflect
import pandas as pd

from cashier import cache


def flatten_event(event):
    ev_type = event["type"]["name"].lower().replace(" ", "_").replace("*","")
    ev_type = ev_type if event["type"]["name"] != "Goal Keeper" else "goalkeeper"
    if ev_type in event:
        for k, v in event[ev_type].items():
            event[f"{ev_type}_{k}"] = v
        del event[ev_type]

    for k, v in event.items():
        if isinstance(v, dict) and "name" in v:
            event[k] = v["name"]
    return event


def filter_and_group_events(events, filters, fmt, flatten):
    events_ = defaultdict(list)
    for ev in events.values():
        ev_type = pluralize(ev["type"]["name"])
        if not is_relevant(ev, filters):
            continue
        if flatten is True or fmt == "dataframe":
            ev = flatten_event(ev)
        events_[ev_type].append(ev)
    return events_


def is_relevant(event, filters):
    return all(event.get("type", {}).get("name") == v for k, v in filters.items())


def reduce_events(all_events: dict, fmt: str) -> dict:
    reduced_events = defaultdict(list)
    for events in all_events:
        for ev_type, evs in events.items():
            reduced_events[ev_type] = reduced_events.get(ev_type, []) + evs
    if fmt == "dataframe":
        for ev_type, evs in reduced_events.items():
            reduced_events[ev_type] = pd.DataFrame(evs)
    return reduced_events


engine = inflect.engine()
@cache()
def pluralize(word):
    word = engine.plural(word)
    word = word.replace("*", "")
    word = word.replace("-", "_")
    word = word.replace(" ", "_")
    word = word.lower()
    return word
